;Copyright (C) 2004-2008 John T. Haller

;Website: http://PortableApps.com/PortableThunderbird

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define PORTABLEAPPNAME "AbiWord Portable"
!define APPNAME "AbiWord"
!define NAME "AbiWordPortable"
!define VER "1.6.0.0"
!define WEBSITE "PortableApps.com/AbiWordPortable"
!define DEFAULTEXE "AbiWord.exe"
!define DEFAULTAPPDIR "AbiSuite2\AbiWord\bin"
!define DEFAULTSETTINGSPATH "settings"
!define DEFAULTSETTINGSFILE "AbiWord.profile"
!define LAUNCHERLANGUAGE "English"

;=== Program Details
Name "${PORTABLEAPPNAME}"
OutFile "..\..\${NAME}.exe"
Caption "${PORTABLEAPPNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${PORTABLEAPPNAME}"
VIAddVersionKey Comments "Allows ${APPNAME} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${PORTABLEAPPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${PORTABLEAPPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
;VIAddVersionKey PrivateBuild ""
;VIAddVersionKey SpecialBuild ""

;=== Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user

; Best Compression
SetCompress Auto
SetCompressor /SOLID lzma
SetCompressorDictSize 32
SetDatablockOptimize On

;=== Include
;(Standard NSIS)
!include FileFunc.nsh
!insertmacro GetRoot
!include Registry.nsh
!include TextFunc.nsh
!insertmacro ConfigRead
!insertmacro ConfigWrite
!insertmacro GetParameters

;(NSIS Plugins)
!include TextReplace.nsh

;(Custom)
!include ReplaceInFileWithTextReplace.nsh
!include ReadINIStrWithDefault.nsh


;=== Program Icon
Icon "..\..\App\AppInfo\appicon.ico"

;=== Icon & Stye ===
;!define MUI_ICON "..\..\App\AppInfo\appicon.ico"

;=== Languages
;!insertmacro MUI_LANGUAGE "${LAUNCHERLANGUAGE}"
LoadLanguageFile "${NSISDIR}\Contrib\Language files\${LAUNCHERLANGUAGE}.nlf"
!include PortableApps.comLauncherLANG_${LAUNCHERLANGUAGE}.nsh

Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var SETTINGSFILE
Var ADDITIONALPARAMETERS
Var EXECSTRING
Var PROGRAMEXECUTABLE
Var INIPATH
Var DISABLESPLASHSCREEN
Var ISDEFAULTDIRECTORY
Var TEMPPATH
Var SECONDARYLAUNCH
Var USERTYPE
Var USERPROFILEPATH
Var CURRENTDRIVE
Var LASTDRIVE
Var MISSINGFILEORPATH
Var APPLANGUAGE


Section "Main"
	ReadEnvStr $USERPROFILEPATH "USERPROFILE"

	;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" NoINI
			StrCpy "$INIPATH" "$EXEDIR\"
			Goto ReadINI

	ReadINI:
		;=== Read the parameters from the INI file
		${ReadINIStrWithDefault} $0 "$INIPATH\${NAME}.ini" "${NAME}" "${APPNAME}Directory" "App\${DEFAULTAPPDIR}"
		StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\$0"
		${ReadINIStrWithDefault} $0 "$INIPATH\${NAME}.ini" "${NAME}" "SettingsDirectory" "Data\${DEFAULTSETTINGSPATH}"
		StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\$0"
		${ReadINIStrWithDefault} $SETTINGSFILE "$INIPATH\${NAME}.ini" "${NAME}" "SettingsFile" "${DEFAULTSETTINGSFILE}"
		${ReadINIStrWithDefault} $ADDITIONALPARAMETERS "$INIPATH\${NAME}.ini" "${NAME}" "AdditionalParameters" ""
		${ReadINIStrWithDefault} $PROGRAMEXECUTABLE "$INIPATH\${NAME}.ini" "${NAME}" "${APPNAME}Executable" "${DEFAULTEXE}"
		${ReadINIStrWithDefault} $DISABLESPLASHSCREEN "$INIPATH\${NAME}.ini" "${NAME}" "DisableSplashScreen" "false"

		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE NoProgramEXE

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy "$ADDITIONALPARAMETERS" ""
		StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
		StrCpy "$DISABLESPLASHSCREEN" "false"
		StrCpy "$SETTINGSFILE" "${DEFAULTSETTINGSFILE}"

		IfFileExists "$EXEDIR\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" NoProgramEXE
			StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\App\${DEFAULTAPPDIR}"
			StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\Data\${DEFAULTSETTINGSPATH}"
			StrCpy "$ISDEFAULTDIRECTORY" "true"
			Goto FoundProgramEXE

	NoProgramEXE:
		;=== Program executable not where expected
		StrCpy $MISSINGFILEORPATH $PROGRAMEXECUTABLE
		MessageBox MB_OK|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
		Abort
		
	FoundProgramEXE:
		;=== Check if already running
		FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
		Pop $R0                     
		StrCmp $R0 "1" "" ShowSplashScreen
			StrCpy $SECONDARYLAUNCH "true"
			Goto CheckForSettings
	
	ShowSplashScreen:
		;=== Show the splash screen before processing the files
		StrCmp $DISABLESPLASHSCREEN "true" CheckForSettings
		InitPluginsDir
		File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"	
		newadvsplash::show /NOUNLOAD 1500 0 0 -1 /L $PLUGINSDIR\splash.jpg
	
	CheckForSettings:
		IfFileExists "$SETTINGSDIRECTORY\$SETTINGSFILE" SettingsFound
		;=== No settings found
		CreateDirectory $SETTINGSDIRECTORY
		StrCmp $ISDEFAULTDIRECTORY "true" CopyDefaultSettings SettingsFound

	CopyDefaultSettings:
		CreateDirectory "$EXEDIR\Data"
		CreateDirectory "$EXEDIR\Data\settings"
		CopyFiles /SILENT $EXEDIR\App\DefaultData\settings\*.* $EXEDIR\Data\settings
		GoTo SettingsFound

	SettingsFound:
		;=== Figure out the temp registry store
		ReadEnvStr $TEMPPATH "TEMP"
		StrCmp $SECONDARYLAUNCH "true" GetPassedParameters

	;AdjustPaths:
		ReadINIStr $LASTDRIVE "$SETTINGSDIRECTORY\${NAME}Settings.ini" "${NAME}Settings" "LastDrive"
		${GetRoot} $EXEDIR $CURRENTDRIVE
		StrCmp $LASTDRIVE $CURRENTDRIVE GetAppLanguage
		IfFileExists "$SETTINGSDIRECTORY\AbiWord.profile" "" RememberPath
			${ReplaceInFile} `$SETTINGSDIRECTORY\AbiWord.profile` `="$LASTDRIVE\` `="$CURRENTDRIVE\`
			Delete "$SETTINGSDIRECTORY\AbiWord.profile.old"
	
	RememberPath:
		WriteINIStr "$SETTINGSDIRECTORY\${NAME}Settings.ini" "${NAME}Settings" "LastDrive" "$CURRENTDRIVE"

	GetAppLanguage:
		ReadEnvStr $APPLANGUAGE "PortableApps.comLanguageCode"
		StrCmp $APPLANGUAGE "" GetPassedParameters ;if not set, move on
		StrCmp $APPLANGUAGE "en" "" FixProfileStrings
			StrCpy $APPLANGUAGE "en-us"
	
	FixProfileStrings:
		${ConfigRead} `$SETTINGSDIRECTORY\AbiWord.profile` `		StringSet="en-US` $0
			StrCmp $0 "" GetCurrentLanguage
				${ConfigWrite} `$SETTINGSDIRECTORY\AbiWord.profile` `		StringSet=` `` $R0
	
	GetCurrentLanguage:
		${ConfigRead} `$SETTINGSDIRECTORY\AbiWord.profile` `		StringSet="` $0
		StrCmp $0 "" "" CheckLanguageChanged ;default is en-us
			StrCmp $APPLANGUAGE "en-us" GetPassedParameters ;keep it blank and move on
				StrCpy $0 `en-us"`
		
	CheckLanguageChanged:
		StrCmp `$APPLANGUAGE"` $0 GetPassedParameters ;if the same, move on
		StrCmp $APPLANGUAGE "en-us" SetAppLanguage ;english is built in, so skip locale file check
		IfFileExists "$PROGRAMDIRECTORY\..\strings\$APPLANGUAGE.strings" SetAppLanguage
			StrCpy $1 $APPLANGUAGE 2
			IfFileExists "$PROGRAMDIRECTORY\..\strings\$1-$1.strings" "" GetPassedParameters
				StrCpy $APPLANGUAGE "$1-$1"

	SetAppLanguage:
		${ConfigRead} `$SETTINGSDIRECTORY\AbiWord.profile` `		StringSet="` $1
		StrCmp $1 "" SetAppLanguageFromBlank
			${ConfigWrite} `$SETTINGSDIRECTORY\AbiWord.profile` `		StringSet="` `$APPLANGUAGE"` $R0
			Goto GetPassedParameters
	
	SetAppLanguageFromBlank:
			${ConfigWrite} `$SETTINGSDIRECTORY\AbiWord.profile` `		name="_custom_` `"$\r$\n		StringSet="$APPLANGUAGE"` $R0

	GetPassedParameters:
		;=== Get any passed parameters
		${GetParameters} $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" --userprofile "$SETTINGSDIRECTORY\$SETTINGSFILE"`
		Goto AdditionalParameters

	LaunchProgramParameters:
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" --userprofile "$SETTINGSDIRECTORY\$SETTINGSFILE" $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" PrepareCustomDictionary

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`
		
	PrepareCustomDictionary:
		StrCmp $SECONDARYLAUNCH "true" StartProgramAndExit
		CreateDirectory "$USERPROFILEPATH\AbiSuite\"
		Rename "$USERPROFILEPATH\AbiSuite\custom.dic" "$USERPROFILEPATH\AbiSuite\customlocal.dic"
		Rename "$USERPROFILEPATH\AbiSuite\AbiCollab.Profile" "$USERPROFILEPATH\AbiSuite\AbiCollab.localProfile"
		CopyFiles "$SETTINGSDIRECTORY\custom.dic" "$USERPROFILEPATH\AbiSuite\"
		CopyFiles "$SETTINGSDIRECTORY\AbiCollab.Profile" "$USERPROFILEPATH\AbiSuite\"

	;LaunchNow:
		;=== Check for registry permissions
		UserInfo::GetAccountType
		Pop $0
		StrCpy $USERTYPE $0
		StrCmp $USERTYPE "Guest" StartProgramNow
		StrCmp $USERTYPE "User" StartProgramNow
		
		${registry::MoveKey} "HKEY_CLASSES_ROOT\.abw" "HKEY_CLASSES_ROOT\.abw-BackupByAbiWordPortable" $R0
		${registry::MoveKey} "HKEY_CLASSES_ROOT\.zabw" "HKEY_CLASSES_ROOT\.zabw-BackupByAbiWordPortable" $R0
		${registry::MoveKey} "HKEY_CLASSES_ROOT\AbiSuite.AbiWord" "HKEY_CLASSES_ROOT\AbiSuite.AbiWord-BackupByAbiWordPortable" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.abw" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.abw-BackupByAbiWordPortable" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.zabw" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.zabw-BackupByAbiWordPortable" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\AbiSuite.AbiWord" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\AbiSuite.AbiWord-BackupByAbiWordPortable" $R0

	StartProgramNow:
		SetOutPath "$PROGRAMDIRECTORY"
		ExecWait $EXECSTRING
		
	CheckRunning:
		Sleep 2000
		FindProcDLL::FindProc "$PROGRAMEXECUTABLE"
		Pop $R0                     
		StrCmp $R0 "1" CheckRunning TheEnd
	
	StartProgramAndExit:
		Exec $EXECSTRING
		Goto CloseRegistry

	TheEnd:
		;=== Cleanup the custom dictionary
		CopyFiles "$USERPROFILEPATH\AbiSuite\custom.dic" "$SETTINGSDIRECTORY\"
		Delete "$USERPROFILEPATH\AbiSuite\custom.dic"
		CopyFiles "$USERPROFILEPATH\AbiSuite\AbiCollab.Profile" "$SETTINGSDIRECTORY\"
		Delete "$USERPROFILEPATH\AbiSuite\AbiCollab.Profile"
		Rename "$USERPROFILEPATH\AbiSuite\customlocal.dic" "$USERPROFILEPATH\AbiSuite\custom.dic"
		Rename "$USERPROFILEPATH\AbiSuite\AbiCollab.localProfile" "$USERPROFILEPATH\AbiSuite\AbiCollab.Profile"
		RMDir "$USERPROFILEPATH\AbiSuite\" ;=== Will only delete if empty (no /r switch)
	
		;=== Restore registry keys
		StrCmp $USERTYPE "Guest" EndWithoutRegRestore
		StrCmp $USERTYPE "User" EndWithoutRegRestore
		${registry::DeleteKey} "HKEY_CLASSES_ROOT\.abw" $R0
		${registry::MoveKey} "HKEY_CLASSES_ROOT\.abw-BackupByAbiWordPortable" "HKEY_CLASSES_ROOT\.abw" $R0
		${registry::DeleteKey} "HKEY_CLASSES_ROOT\.zabw" $R0
		${registry::MoveKey} "HKEY_CLASSES_ROOT\.zabw-BackupByAbiWordPortable" "HKEY_CLASSES_ROOT\.zabw" $R0
		${registry::DeleteKey} "HKEY_CLASSES_ROOT\AbiSuite.AbiWord" $R0
		${registry::MoveKey} "HKEY_CLASSES_ROOT\AbiSuite.AbiWord-BackupByAbiWordPortable" "HKEY_CLASSES_ROOT\AbiSuite.AbiWord" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.abw" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.abw-BackupByAbiWordPortable" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.abw" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.zabw" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.zabw-BackupByAbiWordPortable" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\.zabw" $R0
		${registry::DeleteKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\AbiSuite.AbiWord" $R0
		${registry::MoveKey} "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\AbiSuite.AbiWord-BackupByAbiWordPortable" "HKEY_LOCAL_MACHINE\SOFTWARE\Classes\AbiSuite.AbiWord" $R0
	
	EndWithoutRegRestore:
		StrCmp $DISABLESPLASHSCREEN "true" CleanupRunLocally
			Sleep 2000
			newadvsplash::stop

	CleanupRunLocally:
		${registry::Unload}
		RMDir /r "$TEMPPATH\${NAME}\"
		Goto EndOfLauncher
		
	CloseRegistry:
		${registry::Unload}
	
	EndOfLauncher:
		newadvsplash::stop /WAIT
SectionEnd